package com.fuse.net;

/*
	FUSE Light Server - multiuser server application
	Copyright (C) 2000 Aapo Kyrola / Sulake Oy  Helsinki, Finland

	This program is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 2
	of the License, or (at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/


import java.io.*;
import java.net.*;
import java.util.*;

import com.fuse.*;


/**
  * FUSEConnectionListener is a simple Thread which listens
  * for connections and creates new FUSEConnections for new connections.
  * @author Aapo Kyrol
  */
public class FUSEConnectionListener extends Thread {
	
	private int port;
	private FUSEConnectionManager connectionManager;
	private ServerSocket listenSocket;
	
	
	/**
	  * Constructs new FUSEConnectionListener to given port.
	  * @param port the port to listen
	  * @param maxConnection maximum number of connections
	  */
	public FUSEConnectionListener(FUSEConnectionManager connectionManager, int port, InetAddress bindAddress) {
		super("FUSEConnectionListener port=" + port);
		Log.status("Start connectionListener port=" + port);
		this.port = port;
		this.bindAddress = bindAddress;
		this.connectionManager = connectionManager;
		Log.status("ok..");
		this.start();
	}

	
	public void run() {
		Log.status(this.toString() + " starting..");
		try {
			Log.status("IP=" + InetAddress.getLocalHost());
		} catch (Exception e) {Log.error(e);}
		try {
			listenSocket = new ServerSocket(port, 50, bindAddress);
		
			while(true) {
				Log.status("Accepting connections");
				Socket clientSocket = listenSocket.accept();
				
				/* Start a connection */
				try {
					Log.status("Launching new connection");
					FUSEClientConnection conn = connectionManager.getConnection();
					if (conn != null) {
						conn.setPriority(3);
						conn.handleConnection(clientSocket);
					}
				} catch (NoConnectionsAvailableException ncae) {
					Log.error(new Date().toString() + ": No connections available: connection failed!");
					try {
						PrintWriter out = new PrintWriter(clientSocket.getOutputStream(), true);
						out.println("# ERROR CONNECTION REFUSED = NO CONNECTIONS AVAILABLE ##");
						out.flush();
						clientSocket.close();
					} catch (IOException ioe) {Log.error(ioe);}
				} catch (Exception e) {
					Log.error(e);
					Log.status("Still running");
				}
			}
		
		} catch (IOException ioe) {
			Log.error(ioe);
			System.exit(0);
		}
		
	}
	
	
	public String toString() {
		return "FUSEConnectionListener port=" + port;
	}
	
	private InetAddress bindAddress;
}
